//lifting.c
#include"lifting.h"
#include"TransformPic.h" // for PrintLine(line,N,Level);
void LiftLine(INT16 *line,UINT16 N,\
              INT8 expIn,INT8 expOut){
  //N here refers to half of the line width
  //X0 lowpass filtered values [0,2,4,6,...]
  //X1 highpass                [1,3,5,7,...]
  // 0  1  2  3  4  5       
  // X0 X1 X0 X1 X0 X1 ...
//expRes expC     expAB
//expOut expIn    expIn
//    X1=X1+conv([X0,X0(end)],[alpha,alpha]) (2:end-1)
  ConvAdd(line,N,0,expIn,expIn,expOut);
  //X0=X0+conv([X1(1),X1  ],[beta, beta ]) (2:end-1)
  ConvAdd(line,N,1,expOut,expIn,expOut);
  //X1=X1+conv([X0,X0(end)],[gamma,gamma]) (2:end-1)
  ConvAdd(line,N,2,expOut,expOut,expOut);
  //X0=X0+conv([X1(1),X1  ],[delta,delta]) (2:end-1)
  ConvAdd(line,N,3,expOut,expOut,expOut); 
  UINT16 i;
  INT16 ceta=para(4);
  UINT8 Qceta=Qpara(4);
  for (i=0;i<N;i++){
    line[i*2]  =MulFixExp(line[i*2],  ceta,expOut,Qceta,expOut);//X0=X0*ceta
    line[i*2+1]=DivFixExp(line[i*2+1],ceta,expOut,Qceta,expOut);//X1=X1/ceta
  }
}

void ConvAdd(INT16* line,UINT16 N,UINT8 IndPara,\
             INT8 expAB,INT8 expC,INT8 expOut){
  //uses makro from FixedPoint.h:
  //update(a,b,c,P,expAB,expC,expP,expResult)
  INT16 p=para(IndPara);//alpha,beta,gamma,delta
  UINT8 Qp=Qpara(IndPara);
  UINT16 i;
  if ((IndPara==0)||(IndPara==2)){
    //update highpass: line[1,3,5,...], P=alpha,gamma
    for (i=0;i<N-1;i++){
      //line[i*2+1]+=(line[i*2]+line[i*2+2])*P
      line[i*2+1]=update(line[i*2],line[i*2+2],line[i*2+1],p,	\
                         expAB,expC,Qp,expOut);
    }
    i=N-1;
    //line[i*2+1]+=2*line[i*2]*P symm.extension right end
    line[i*2+1]=update(line[i*2],line[i*2],line[i*2+1],p,	\
                       expAB,expC,Qp,expOut);
  }	  
  else{
    //update lowpass: line[0,2,4,...], P=beta,ceta
    i=0;
    //line[i*2]+=2*line[2*i+1]*P symm.extension left end
    line[i*2]=update(line[i*2+1],line[i*2+1],line[i*2],p,	\
		     expAB,expC,Qp,expOut);
    for (i=1;i<N;i++){
      //line[i*2]+=(line[i*2-1]+line[i*2+1])*P
      line[i*2]=update(line[i*2-1],line[i*2+1],line[i*2],p,	\
		       expAB,expC,Qp,expOut);}
  }
}

INT16 para(UINT8 index){
  switch(index){
  case 0:return -25987;//alpha in Q14
  case 1:return -27777;//beta  in Q19
  case 2:return  28931;//gamma in Q15
  case 3:return  29066;//delta in Q16
  case 4:return  18835;//ceta  in Q14
  default:\
    printf("asked for wrong lifting parameter\n");
    return 0;
  }

}


UINT8 Qpara(UINT8 index){
  switch(index){
  case 0:return 14;//alpha in Q14
  case 1:return 19;//beta  in Q19
  case 2:return 15;//gamma in Q15
  case 3:return 16;//delta in Q16
  case 4:return 14;//ceta  in Q14
  default:\
    printf("asked for wrong Q parameter\n");
    return 0;
  }

}
